# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

import puwrapper_pb2 as puwrapper__pb2


class PURequestStub(object):
  """The pu request service definition.
  """

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.Request = channel.unary_unary(
        '/puwrapper.PURequest/Request',
        request_serializer=puwrapper__pb2.RequestMsg.SerializeToString,
        response_deserializer=puwrapper__pb2.RequestRetMsg.FromString,
        )


class PURequestServicer(object):
  """The pu request service definition.
  """

  def Request(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_PURequestServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'Request': grpc.unary_unary_rpc_method_handler(
          servicer.Request,
          request_deserializer=puwrapper__pb2.RequestMsg.FromString,
          response_serializer=puwrapper__pb2.RequestRetMsg.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'puwrapper.PURequest', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))
