# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: puwrapper.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='puwrapper.proto',
  package='puwrapper',
  syntax='proto3',
  serialized_options=_b('\n\024com.daline.puwrapperB\014RequestProtoP\001'),
  serialized_pb=_b('\n\x0fpuwrapper.proto\x12\tpuwrapper\"\x1c\n\nRequestMsg\x12\x0e\n\x06strReq\x18\x01 \x01(\t\"\x1f\n\rRequestRetMsg\x12\x0e\n\x06strRet\x18\x01 \x01(\t2I\n\tPURequest\x12<\n\x07Request\x12\x15.puwrapper.RequestMsg\x1a\x18.puwrapper.RequestRetMsg\"\x00\x42&\n\x14\x63om.daline.puwrapperB\x0cRequestProtoP\x01\x62\x06proto3')
)




_REQUESTMSG = _descriptor.Descriptor(
  name='RequestMsg',
  full_name='puwrapper.RequestMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='strReq', full_name='puwrapper.RequestMsg.strReq', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=30,
  serialized_end=58,
)


_REQUESTRETMSG = _descriptor.Descriptor(
  name='RequestRetMsg',
  full_name='puwrapper.RequestRetMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='strRet', full_name='puwrapper.RequestRetMsg.strRet', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=60,
  serialized_end=91,
)

DESCRIPTOR.message_types_by_name['RequestMsg'] = _REQUESTMSG
DESCRIPTOR.message_types_by_name['RequestRetMsg'] = _REQUESTRETMSG
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

RequestMsg = _reflection.GeneratedProtocolMessageType('RequestMsg', (_message.Message,), {
  'DESCRIPTOR' : _REQUESTMSG,
  '__module__' : 'puwrapper_pb2'
  # @@protoc_insertion_point(class_scope:puwrapper.RequestMsg)
  })
_sym_db.RegisterMessage(RequestMsg)

RequestRetMsg = _reflection.GeneratedProtocolMessageType('RequestRetMsg', (_message.Message,), {
  'DESCRIPTOR' : _REQUESTRETMSG,
  '__module__' : 'puwrapper_pb2'
  # @@protoc_insertion_point(class_scope:puwrapper.RequestRetMsg)
  })
_sym_db.RegisterMessage(RequestRetMsg)


DESCRIPTOR._options = None

_PUREQUEST = _descriptor.ServiceDescriptor(
  name='PURequest',
  full_name='puwrapper.PURequest',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=93,
  serialized_end=166,
  methods=[
  _descriptor.MethodDescriptor(
    name='Request',
    full_name='puwrapper.PURequest.Request',
    index=0,
    containing_service=None,
    input_type=_REQUESTMSG,
    output_type=_REQUESTRETMSG,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_PUREQUEST)

DESCRIPTOR.services_by_name['PURequest'] = _PUREQUEST

# @@protoc_insertion_point(module_scope)
