import py_compile
import shutil

py_compile.compile('DalBaseUnit.py', r'..\DalineUnit\DalBaseUnit.pyc')
py_compile.compile('OracleOper.py', r'..\DalineUnit\OracleOper.pyc')
py_compile.compile('puwrapper_pb2.py', r'..\DalineUnit\puwrapper_pb2.pyc')
py_compile.compile('puwrapper_pb2_grpc.py', r'..\DalineUnit\puwrapper_pb2_grpc.pyc')
py_compile.compile('OracleOper.py', r'..\DalineUnit\OracleOper.pyc')
py_compile.compile('Template.py', r'..\DalineUnit\Template.pyc')
py_compile.compile('DalDecorate.py', r'..\DalineUnit\DalDecorate.pyc')
py_compile.compile('DalGenScript.py', r'..\DalineUnit\DalGenScript.pyc')
py_compile.compile('DesignFile.py', r'..\DalineUnit\DesignFile.pyc')
py_compile.compile('GenTestData.py', r'..\DalineUnit\GenTestData.pyc')
